/**************************************************/
/* author	Tony Madden	(tm@mqware.com) */
/**************************************************/
#ifndef MAXCONN_H
#define	MAXCONN_H

#include	<string.h>

#include	<cmqc.h>

#define	MXC_QUEUE_NAME_PREFIX					"MAXCONN."

#define	MXC_BOOTSTRAP_PREFIX					'B'
#define	MXC_VOLATILE_PREFIX						'V'

#define MXC_BOOTSTRAP							1
#define	MXC_VOLATILE							2

#define	MXC_END									0
#define	MXC_MORE								1

#ifdef MQAT_WINDOWS_NT
	#define	MXC_DEFAULT_LOG_FILE_DIRECTORY		"c:"
	#define	MXC_ROOT_LOCATION					"c:\\"
	#define MXC_FILE_DELIMITER					"\\"		
#else
	#define	MXC_DEFAULT_LOG_FILE_DIRECTORY		""
	#define	MXC_ROOT_LOCATION					""
	#define MXC_FILE_DELIMITER					"/"
#endif
#define	MXC_DEFAULT_LOG_FILE_PREFIX				"MAXCONN"
#define	MXC_DEFAULT_LOG_FILE_SUFFIX				"LOG"

#define	MXC_DEFAULT_MAX_OBJECTS_IN_USE_ATTEMPTS	20

#define	MXC_DEFAULT_MODEL_QUEUE_NAME			"MAXCONN.MODEL.QUEUE"
#define	MXC_DEFAULT_EVENT_QUEUE_NAME			"MAXCONN.EVENT.QUEUE"	

#define	MXC_CHANNEL_ACTIVATED					1
#define	MXC_CHANNEL_DEACTIVATED					2

#define	MXC_EVENTS_NOT_ACTIVE					0
#define	MXC_EVENTS_ACTIVE						1
#define	MXC_EVENTS_MQ_ACTIVE					3
#define	MXC_EVENTS_FIRST_LAST					4

#define MXC_LIT_EVENTS_NOT_ACTIVE				"no"
#define	MXC_LIT_EVENTS_ACTIVE					"yes"
#define	MXC_LIT_EVENTS_MQ_ACTIVE				"mq"

#define	MXC_EVENT_STATE_OK						0
#define	MXC_EVENT_STATE_FIRST					2
#define	MXC_EVENT_STATE_LAST					4
#define	MXC_EVENT_STATE_WARN					8
#define	MXC_EVENT_STATE_MAX						16
#define	MXC_EVENT_STATE_UNKNOWN					32


#define	MXC_EVENT_STATE_FRST_TEXT				"EVENT=FRST"
#define	MXC_EVENT_STATE_LAST_TEXT				"EVENT=LAST"
#define	MXC_EVENT_STATE_OK_TEXT					"EVENT=OK  "
#define	MXC_EVENT_STATE_WARN_TEXT				"EVENT=WARN"
#define	MXC_EVENT_STATE_MAX_TEXT				"EVENT=MAX "

#define	MXC_CONNECTION_REQUEST_TEXT				"REQU"
#define	MXC_CONNECTION_TERMINATION_TEXT			"TERM"

#define	MXC_CONNECTION_REQUEST					1
#define	MXC_CONNECTION_TERMINATION				2

#define	MXC_DELIM	','

#define	MXC_TRACE_LEVEL_NONE					0
#define	MXC_TRACE_LEVEL_INFO					1
#define	MXC_TRACE_LEVEL_DEBUG					3

#define	MXC_TRACE_LEVEL_INFO_TEXT				"info"
#define	MXC_TRACE_LEVEL_DEBUG_TEXT				"debug"

#define	MXC_REJECTED							'r'
#define	MXC_ACCEPTED							'a'

#define	MXC_BROWSE								1
#define	MXC_GET									2

#define	MXC_EXITUSERAREA_RESPONSE_OFFSET		0
#define	MXC_EXITUSERAREA_TRACELEVEL_OFFSET		1

#define	MXC_DEFAULT_IN_USE_WAIT_MSECS			200

//Return Codes
#define	MXC_RC_NONE								0
#define	MXC_RC_ALLOW_CONNECTION					0
#define	MXC_RC_REFUSE_CONNECTION				1
#define	MXC_RC_NO_DATA_AVAILABLE				4
#define	MXC_RC_ZERO_CHANNEL_INSTANCES			5
#define	MXC_RC_ERROR							8

// reason codes used in PCF event messages.
// Values are calculated to be in the middle of the MQRC_APPL_ range.
#define	MXC_PCF_REASON_CONNECTIONS_OK			140;
#define	MXC_PCF_REASON_WARN_THRESHOLD_REACHED	141;
#define	MXC_PCF_REASON_MAX_CONNECTIONS_REACHED	142;

//command line parameters
#define	MXC_PARM_HELP							"-?"
#define	MXC_PARM_ONLY_CLEAR_SPECIFIC_MESSAGES	"-s"
#define	MXC_PARM_SVRCONN_CHANNEL_NAME			"-sc"
#define	MXC_PARM_DISPLAY						"-d"
#define	MXC_PARM_LOAD							"-l"	
#define	MXC_PARM_PARAMETER_FILE					"-pf"
#define	MXC_PARM_QUEUE_NAME						"-q"
#define	MXC_PARM_QMGR_NAME						"-qm"

// parm file parameters
#define	MXC_PARMFILE_EVENT_QUEUE_NAME				"EventQueue"
#define	MXC_PARMFILE_FIRST_LAST_EVENTS				"FirstLastEvents"	
#define	MXC_PARMFILE_LOG_FILE_DIRECTORY				"LogFileDirectory"
#define	MXC_PARMFILE_LOG_FILE_NAME					"LogFileName"
#define	MXC_PARMFILE_MAX_NUMBER_OF_CONNECTIONS		"MaxConnections"
#define	MXC_PARMFILE_MAX_NUMBER_OF_IN_USE_ATTEMPTS	"MaxInUseAttempts"
#define	MXC_PARMFILE_EVENT_FLAG						"EventFlag"
#define	MXC_PARMFILE_FREQUENCY_OF_PCF_CHECKS		"PcfCheckRatio"
#define	MXC_PARMFILE_SVRCONN_CHANNEL_NAME			"ServerConnChannel"
#define	MXC_PARMFILE_TRACE_LEVEL					"TraceLevel"
#define	MXC_PARMFILE_EVENT_WARN_THRESHOLD			"WarnThreshold"

#define	MXC_PARAMETER_FILE_SET_ARGUMENT_NAME		"ChannelParameters:"

#define	MXC_YES									"yes"
#define	MXC_NO									"no"

#define	MXC_TERM								1
#define	MXC_REQUEST								2

#define	MXC_LOG_DEBUG							"DEBUG"
#define MXC_LOG_INFO							"INFO "
#define MXC_LOG_EVENT							"EVENT"
#define MXC_LOG_ERROR							"ERROR"

#define	MXC_STATE_DISPLAY						1
#define	MXC_STATE_LOAD							2	

#define	MXC_BROWSE								1
#define	MXC_GET_PUT								2

#define MXC_DOTS								"..................................................";
#define	MXC_DOT_OFFSET							35


//**********************************************************************************************************
// STATEDATA structure which holds the State information 
typedef struct	{

	long	lMaxConnections ;
	long	lExistingConnections ;
	long	lTraceLevel;
	long	lMaxNumberOfInUseAttempts;
	long	lFrequencyOfPCFChecks;
	long	lTimesAtConnMaxSinceLastPCFCheck;
	long	lEventFlag;
	long	lEventState;
	long	lWarnThreshold;

	char	szLogFileDirectory[257];
	char	szLogFileName[257];
	char	szEventQueue[MQ_Q_NAME_LENGTH+1];
	char	cStateDataType;
} MXC_STATEDATA,*pMXC_STATEDATA;

//**********************************************************************************************************
// PCFSTRINGHEADER structure. Used when constructing MQ event messages
typedef struct  {
	char	eventType[10];
	char	eventContext[4];
} MXC_PCFSTRINGHEADER,*pMXC_PCFSTRINGHEADER;

//**********************************************************************************************************
// PCFSTRINGVARIABLEHEADER structure. Used when constructing MQ event messages
typedef struct {
	char	varLength[3];
} MXC_PCFSTRINGVARIABLEHEADER,*pMXC_PCFSTRINGVARIABLEHEADER;

#define MXC_PCFSTRINGVARIABLEHEADER_LENGTH_DEFAULT  "000"

//**********************************************************************************************************
// the following functions are used both by the maxconn exit (C source) and the maxconnload utility (C++ source)
#ifdef __cplusplus
	extern "C" {
#endif

long	createStateDataString(MXC_STATEDATA,char[],char);
long	extractFromStateDataString(pMXC_STATEDATA,char*,long,char[]);
long	isNumeric(char []);
long	next(char*,long*,long ,char,char[],long);

#ifdef __cplusplus
	}
#endif

#endif

