//**********************************************************************************************************
// Creates and parses maxconn State delimited strings
//
// author	Tony Madden	(tm@mqware.com)
//
//
//
//**********************************************************************************************************
// Fixes
//
// V 1.0.1	FIX001	incorrect address used when parsing state data delimited string
//**********************************************************************************************************
#include	<maxconn.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

//**********************************************************************************************************
// Convert a STATEDATA variable into a string
//
// arg_sd			STATEDATA variable
// arg_szString		array which will contain a delimited string
// arg_cType		type of state data (bootstrap or volatile)
//
// return			zero
//**********************************************************************************************************
long	createStateDataString(MXC_STATEDATA arg_sd,char arg_szString[],char	arg_cType){
	sprintf(arg_szString,"%c%c%ld%c%ld%c%ld%c%ld%c%ld%c%ld%c%ld%c%ld%c%ld%c%s%c%s%c%s%c",
												arg_cType,
												MXC_DELIM,
								 				arg_sd.lMaxConnections,
												MXC_DELIM,
												arg_sd.lExistingConnections,
												MXC_DELIM,
												arg_sd.lMaxNumberOfInUseAttempts,
												MXC_DELIM,
												arg_sd.lTimesAtConnMaxSinceLastPCFCheck,
												MXC_DELIM,
												arg_sd.lFrequencyOfPCFChecks,
												MXC_DELIM,
							 	 				arg_sd.lEventFlag,
												MXC_DELIM,
												arg_sd.lWarnThreshold,
												MXC_DELIM,
												arg_sd.lEventState,
												MXC_DELIM,
												arg_sd.lTraceLevel,
												MXC_DELIM,
												arg_sd.szLogFileDirectory,
												MXC_DELIM,
												arg_sd.szLogFileName,
												MXC_DELIM,
												arg_sd.szEventQueue,
												MXC_DELIM   // NOTE - must have trailing delimiter
												);

	return 0;
}

//**********************************************************************************************************
// Parse a delimited string and populate a STATEDATA variable with the values
//
// arg_pSd				STATEDATA variable
// arg_pcBuffer			buffer containing delimited data
// arg_lBufferLength	buffer length
// arg_szError			array which will be populated with an error message (if necessary)	
//
// return				zero or error code
//**********************************************************************************************************
long	extractFromStateDataString(pMXC_STATEDATA arg_pSd ,char* arg_pcBuffer,long arg_lBufferLength,char	arg_szError[]){

	long	lNumberOfElements=0;
	long	lRc=0;
	long	lIsNext;
	long	lOffset = 0;
	long	lIncomplete = 0;
	char	szTemp[500];

	#define	EFDSD_MAX_SIZE_FOR_LONG		10		

	FILE* file;
	char xxx[500];
	file = fopen("c:\\temp\\exitfile.txt","a+");
	if (file != 0){
		//sprintf(xxx,"%offset %ld, lDelimitedValueLength %ld, delim %c, buff %s",*arg_plOffset,lDelimitedValueLength,arg_cDelim,arg_pcDelimited);
		sprintf(xxx,"delim %c",MXC_DELIM);
		fputs(xxx,file);
		fputs("\n",file);
		fclose(file);
	}

	do{
		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,1);
		arg_pSd->cStateDataType = szTemp[0];
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lMaxConnections = atol(szTemp);
		}else{
			sprintf(arg_szError,"MaxConnections in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}

		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lExistingConnections = atol(szTemp);
		}else{
			sprintf(arg_szError,"Existing Connections in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lMaxNumberOfInUseAttempts = atol(szTemp);
		}else{
			sprintf(arg_szError,"MaxNumberOfInUseAttempts in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lTimesAtConnMaxSinceLastPCFCheck = atol(szTemp);
		}else{
			sprintf(arg_szError,"lTimesAtConnMaxSinceLastPCFCheck in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lFrequencyOfPCFChecks = atol(szTemp);
		}else{
			sprintf(arg_szError,"FrequencyOfPCFChecks in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}
	

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lEventFlag = atol(szTemp);
		}else{
			sprintf(arg_szError,"EventFlag in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lWarnThreshold = atol(szTemp);
		}else{
			sprintf(arg_szError,"WarnThreshold in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lEventState = atol(szTemp);
		}else{
			sprintf(arg_szError,"EventState in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,szTemp,EFDSD_MAX_SIZE_FOR_LONG);
		if (isNumeric(szTemp)){
			arg_pSd->lTraceLevel = atol(szTemp);
		}else{
			sprintf(arg_szError,"TraceLevel in delimited state data is not numeric: <%s>",szTemp);
			lRc = MXC_RC_ERROR;
			break;
		}
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		
		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,arg_pSd->szLogFileDirectory,sizeof(arg_pSd->szLogFileDirectory)-1);
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		lIsNext = next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,arg_pSd->szLogFileName,sizeof(arg_pSd->szLogFileName)-1);
		if (!lIsNext){
			lIncomplete = 1;
			break;
		}

		next(arg_pcBuffer,&lOffset,arg_lBufferLength,MXC_DELIM,arg_pSd->szEventQueue,sizeof(MQ_Q_NAME_LENGTH));

		break;
	}while(0);

	if (lIncomplete){
		char	szError[1000];
		strncpy(szError,arg_pcBuffer,sizeof(szError)-1);
		szError[sizeof(szError)-1] = 0;
		sprintf(arg_szError,"Incomplete State data String: <%s>",szError);
		lRc = MXC_RC_ERROR;
	}

	return lRc;
}



//**********************************************************************************************************
// Check whether a string contains only numeric bytes, or not
//
// arg_szString			the string value
//
// return				0 for numeric, 1 for not numeric
//**********************************************************************************************************
long	isNumeric(char arg_szString[]){
	unsigned int i;

	if (arg_szString[0] == '+' || arg_szString[0] == '-'){
		i = 1;
	}else{
		i=0;
	}

	for(;i < strlen(arg_szString);i++){
		if (!isdigit(arg_szString[i])){
			return 0;
		}
	}

	return 1;
}

//**********************************************************************************************************
// Parser routine. Get the next delimited value from delimited data.
//
// arg_pcDelimited			delimited data
// arg_plOffset				offset into delimited data
// arg_lTotalLength			total length of delimited data
// arg_cDelim				delimiter value
// arg_szValue				array containing the latest delimited value
// arg_lMaxDataSize			max size for the current delimited value
//
// return					MXC_END if no more delimited values, or MXC_MORE if there are more delimited values
//**********************************************************************************************************
long	next(char* arg_pcDelimited,long*	arg_plOffset,long	arg_lTotalLength,char arg_cDelim,char arg_szValue[],long	arg_lMaxDataSize){
	char*	pcDelim;
	char*	pcStart;
	long	lRc;
	long	lDelimitedValueLength;
	long	lMaxLength;

	// pcStart is the address at which to start searching for next delimiter
	pcStart = arg_pcDelimited + *arg_plOffset;

	// maxLength is the length from pcStart to end of delimited data
	lMaxLength = arg_lTotalLength - *arg_plOffset;

	// Find a delimiter...
															
	pcDelim = (char*)memchr(pcStart,arg_cDelim,lMaxLength);	//FIX001  replace arg_pcDelimited with pcStart in folling line
	if (pcDelim){
		// Found a delimiter, so calculate length of data prior to delimiter
		lDelimitedValueLength = (long)(pcDelim - pcStart);
		if (lMaxLength < lDelimitedValueLength){
			lDelimitedValueLength = lMaxLength;
		}
		// Copy delimited value to szValue and null terminate it
		memcpy(arg_szValue,pcStart,lDelimitedValueLength);

		arg_szValue[lDelimitedValueLength] = 0;
		// Recalculate offset

	
		(*arg_plOffset) += lDelimitedValueLength + 1;
		lRc = MXC_MORE;

	}else{
		// Copy to the end of delimited data
		memcpy(arg_szValue,arg_pcDelimited,arg_lMaxDataSize);
		arg_szValue[arg_lMaxDataSize] = 0;
		lRc = MXC_END;
	}
	return lRc;

}

