//***************************************************************************
//
// X_ArgumentParser accepts arguments and parses them.
// Arguments can be presented in main form (int,char**), as a string or in a file.
//
// Arguments can be parsed as any one of the following:
//				switches (first byte of switch must be '-'). Switches can have an associated value . 
//				name-value pairs
//				individual arguments 
//
//
// author	Tony Madden	(tm@mqware.com)
//***************************************************************************

#ifndef X_ARGUMENT_PARSER_DEFINED
#define X_ARGUMENT_PARSER_DEFINED

#define X_ARGUMENT_PARSER_VERSION		"1.0.1"

#include	<string>
#include	<map>
#include	<vector>
#include	<cerrno>
#include	<fstream>
#include	<sstream>

#include	<x_exception.hpp>
#include	<x_string.hpp>
#include	<x_tokenizer.hpp>

#define		_AP_NAME_VALUE_PAIRS	1
#define		_AP_SWITCHES			2
#define		_AP_SINGLES				3

#define		_AP_QUOTE				'\"'
#define		_AP_SWITCH_PREFIX		'-'
#define		_AP_COMMENT				'#'

#define		_AP_INTERNAL_SET_DELIMITER	0x01

using namespace std;


class X_ArgumentParserMetaData{
private:
	long	lReferenceCount;
	long	lInstanceCount;

	void	initVariables();
public:
	X_ArgumentParserMetaData();

	bool			decrementReferenceCount();
	long			getInstanceCount();
	long			getReferenceCount();
	void			incrementInstanceCount();
	void			incrementReferenceCount();
	
};

typedef	map<string,X_ArgumentParserMetaData*,x_TrueIfLessThanString> X_StrMetaMap ;


class	X_ArgumentParser{

private:
	string	sParameters;
	char	cQuote;

	long	lParseType;

	char	cComment;

	bool	bCommentsActivated;
	bool	bIgnoreCase;
	bool	bEscapeCharacter;
	bool	bSet;

	vector<string>	vSpaceCandidates;

	X_StrVstrMap	mNameValues;

	X_StrMetaMap	mNameValuesMetaData;


	vector<string>	vOriginalArguments;


	void					appendStringVector(vector<string>*,const vector<string>&);
	long					getInternal(string,vector<string>*,bool,long);
	long					getInstanceCount(string);
	void					initializeValues();
	void					initializeLocals();
	void					internalFileLoad(string, string* ,vector<string>*);
	void					internalLoadFromString(const string&,vector<string>*,char);
	string					removeEscapeCharacters(string s);
	
	void					setNameValuePairs(bool);
	
public:
	X_ArgumentParser(long,bool);
	X_ArgumentParser(long,char);
	~X_ArgumentParser(){};

	long					exists(string);
	long					get(string,string*) ;
	long					get(string,vector<string>*);
	string					get(string);
	long					get(string,string*,bool);
	long					get(string,vector<string>*,bool);
	void					getNameValuesAsMap(X_StrVstrMap*);
	void					getArgumentsAsVector(vector<string>*);
	char					getQuoteValue();
	bool					getReferenceCount(string,long*);
	bool					getUnusedNames(vector<string>*);
	static string			getVersion();
	bool					isUnusedNames();
	void					loadArgumentsFromFile(string);
	void					loadArgumentSetsFromFile(string,string);
	void					loadArgumentsFromMain(long,char**);
	void					loadArgumentsFromString(const string&);
	long					numberOfInstances(string) ;
	long					numberOfInstances(string,long);
	void					parse();
	void					parse(bool,bool);
	void					reset();
	void					setQuoteValue(char);
	string					toString();
};



#endif

