//***********************************************************************************************************
//
// MQ numeric constant conversion class
//
//
// author	Tony Madden	(tm@mqware.com)
//***********************************************************************************************************

#include	<x_mq.hpp>
#include	<x_string.hpp>

extern "C" {
	int compareCode (const void * arg_pvElement1, const void * arg_pvElement2);
}

int compareCode (const void * arg_pvElement1, const void * arg_pvElement2){
	return *(const long*)arg_pvElement1-*(const long*)arg_pvElement2;
}

bool X_Mq::numberToString(char*	arg_pcTable,long	arg_lTableSize,long	arg_lNumber,string* arg_psString)
{
	//***********************************************************************
	//* Search the return code table for a match
	//***********************************************************************
	pXMQ_CONVERSION_TABLE	pCT;
	XMQ_CONVERSION_TABLE		cT;
	string		sString;
	bool		bRc;


	//* Get the offset (makes the code portable between differing struct alignments)
	long lNumberOffset = (long)((char*)&(cT.reason) - (char*)&(cT.string));

	//***********************************************************************
	//* The return code table has the numeric value as second field (Code), 
	//* hence the offset by the size (TEXT_MQ_REASON_SIZE) of the first field (Text) 
	//************************************************************************
	if ( pCT = (pXMQ_CONVERSION_TABLE)bsearch(&arg_lNumber,
										(char*)(arg_pcTable) + lNumberOffset,
										arg_lTableSize/sizeof(XMQ_CONVERSION_TABLE),
										sizeof(XMQ_CONVERSION_TABLE),
										compareCode) ) {
		pCT = (pXMQ_CONVERSION_TABLE)(((char*)pCT) - lNumberOffset);

		string s = string(pCT->string);
		*arg_psString = X_String::trim(s) ;
		bRc = true;
	}else {
		ostringstream	ost;
		ost << "unknown ("  <<  arg_lNumber << ")";
		*arg_psString = ost.str();
		bRc = false;
	}
	return bRc;
}

bool	X_Mq::stringToNumber(char* arg_pcTable,long arg_lTableSize,long* arg_plNumber,string arg_sString){
	pXMQ_CONVERSION_TABLE pCT;
	int	i;
	bool	bRc = false;

	pCT = (pXMQ_CONVERSION_TABLE)arg_pcTable;

	for (i = 0; i < arg_lTableSize/sizeof(XMQ_CONVERSION_TABLE);i++){
		if (arg_sString == (pCT + i)->string){
			*arg_plNumber = (pCT+i)->reason;
			bRc = true;
			break;
		}
	}

	return bRc;
}
//****************************************************************************************************
string	X_Mq::convertBitwiseNumberToString(char* arg_pcTable,long arg_lTableSize,long arg_lBitwise,long arg_lNumber){
	string s;
	if ((arg_lBitwise & arg_lNumber) == arg_lNumber){
		numberToString(arg_pcTable,arg_lTableSize,arg_lNumber,&s);
	}
	return s;
}

bool	X_Mq::convertBitwiseNumberToString(char* arg_pcTable,long arg_lTableSize,long arg_lBitwise,long arg_lNumber,string* arg_psString){
	bool	bRc = false;

	if ((arg_lBitwise & arg_lNumber) == arg_lNumber){
		
		bRc = numberToString(arg_pcTable,arg_lTableSize,arg_lNumber,arg_psString);
	}
	return bRc;
}

//****************************************************************************************************
string X_Mq::convertToStringMQRC(long	arg_lReason){
	string s;
	numberToString((char*)&CT_MQRC,sizeof(CT_MQRC),arg_lReason,&s);
	return s;
}
bool X_Mq::convertToStringMQRC(long	arg_lReason,string* arg_psString){
	return numberToString((char*)&CT_MQRC,sizeof(CT_MQRC),arg_lReason,arg_psString);
}

//****************************************************************************************************
string X_Mq::convertToStringMQOO(long	arg_lNumber){
	
	
	string s;
	convertToStringMQOO(arg_lNumber,&s);
	return s;
}

//****************************************************************************************************
bool X_Mq::convertToStringMQOO(long	arg_lNumber,string* arg_psString){
	bool bRc = false;
	string s;

	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_INPUT_AS_Q_DEF,&s)){ 
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_INPUT_SHARED,&s)){     
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}      
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_INPUT_EXCLUSIVE ,&s)){   
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}    
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_BROWSE,&s )){	  
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}				
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_OUTPUT,&s)){  
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}               
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_INQUIRE,&s)){  
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}              
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_SET,&s)){  
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}                  
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_BIND_ON_OPEN,&s)){   
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}        
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_SAVE_ALL_CONTEXT,&s)){   
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}    
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_PASS_IDENTITY_CONTEXT,&s)){  
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_PASS_ALL_CONTEXT,&s)){    
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}   
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_SET_IDENTITY_CONTEXT,&s)){   
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_SET_ALL_CONTEXT,&s)){      
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}  
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_ALTERNATE_USER_AUTHORITY,&s)){ 
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_FAIL_IF_QUIESCING,&s)){   
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}       

	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_RESOLVE_NAMES,&s)){    
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}

	if (convertBitwiseNumberToString((char*)&CT_MQOO,sizeof(CT_MQOO),arg_lNumber,MQOO_BIND_NOT_FIXED,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}else{
		*arg_psString += "MQOO_BIND_AS_Q_DEF";
		*arg_psString +=  XMQ_BITWISE_DELIMITER;
	}

	if ( (*arg_psString)[arg_psString->size()-1] == XMQ_BITWISE_DELIMITER){
		arg_psString->resize(arg_psString->size()-1);
	}

	return bRc;
}
long X_Mq::convertToNumberMQOO(string s){
	long r;
	stringToNumber((char*)&CT_MQOO,sizeof(CT_MQOO),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQOO(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQOO,sizeof(CT_MQOO),pReason,sString);
}

//****************************************************************************************************
// MQPER conversions
//****************************************************************************************************
string X_Mq::convertToStringMQPER(long	reason){
	string s;
	numberToString((char*)&CT_MQPER,sizeof(CT_MQPER),reason,&s);
	return s;
}
bool X_Mq::convertToStringMQPER(long	reason,string* arg_psString){
	return numberToString((char*)&CT_MQPER,sizeof(CT_MQPER),reason,arg_psString);
}
long X_Mq::convertToNumberMQPER(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQPER,sizeof(CT_MQPER),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQPER(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQPER,sizeof(CT_MQPER),pReason,sString);
}

//****************************************************************************************************
// MQQT conversions
//****************************************************************************************************
string X_Mq::convertToStringMQQT(long	reason){
	string s;
	numberToString((char*)&CT_MQQT,sizeof(CT_MQQT),reason,&s);
	return s;
}
bool X_Mq::convertToStringMQQT(long	reason,string* arg_psString){
	return numberToString((char*)&CT_MQQT,sizeof(CT_MQQT),reason,arg_psString);
}
long X_Mq::convertToNumberMQQT(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQQT,sizeof(CT_MQQT),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQQT(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQQT,sizeof(CT_MQQT),pReason,sString);
}

//****************************************************************************************************
// MQCMD conversions
//****************************************************************************************************
string X_Mq::convertToStringMQCMD(long	command){
	string s;
	numberToString((char*)&CT_MQCMD,sizeof(CT_MQCMD),command,&s);
	return s;
}

bool X_Mq::convertToStringMQCMD(long	command,string* arg_psString){
	return numberToString((char*)&CT_MQCMD,sizeof(CT_MQCMD),command,arg_psString);

}
long X_Mq::convertToNumberMQCMD(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQCMD,sizeof(CT_MQCMD),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQCMD(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQCMD,sizeof(CT_MQCMD),pReason,sString);
}

//****************************************************************************************************
// MQAT conversions
//****************************************************************************************************
string X_Mq::convertToStringMQAT(long appType){
	string s;
	if (appType >= MQAT_USER_FIRST && appType <= MQAT_USER_LAST){
		ostringstream	ost;
		ost << "User value: " << appType;
		s = ost.str();
	}else{
		numberToString((char*)&CT_MQAT,sizeof(CT_MQAT),appType,&s);
	}
	return s;
}

bool X_Mq::convertToStringMQAT(long appType,string* arg_psString){
	bool	bRc;
	if (appType >= MQAT_USER_FIRST && appType <= MQAT_USER_LAST){
		ostringstream	ost;
		ost << "User value: " << appType;
		*arg_psString = ost.str();
		bRc = false;
	}else{
		bRc = numberToString((char*)&CT_MQAT,sizeof(CT_MQAT),appType,arg_psString);
	}
	return bRc;
}
long X_Mq::convertToNumberMQAT(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQAT,sizeof(CT_MQAT),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQAT(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQAT,sizeof(CT_MQAT),pReason,sString);
}

//****************************************************************************************************
// MQMT conversions
//****************************************************************************************************
string X_Mq::convertToStringMQMT(long appType){
	string s;
	convertToStringMQMT(appType,&s);
	return s;
}

bool X_Mq::convertToStringMQMT(long appType,string* arg_psString){
	bool	bRc;
	if (appType >= MQMT_APPL_FIRST && appType <= MQMT_APPL_LAST){
		ostringstream	ost;
		ost << "Appl value: " << appType;
		*arg_psString = ost.str();
		bRc =  false;
	}else{
		bRc = numberToString((char*)&CT_MQMT,sizeof(CT_MQMT),appType,arg_psString);
	}
	return bRc;
}
long X_Mq::convertToNumberMQMT(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQMT,sizeof(CT_MQMT),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQMT(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQMT,sizeof(CT_MQMT),pReason,sString);
}
//****************************************************************************************************
// MQFB conversions
//****************************************************************************************************
string X_Mq::convertToStringMQFB(long appType){
	string s;
	
	convertToStringMQFB(appType,&s);
	
	return s;
}

bool X_Mq::convertToStringMQFB(long appType,string* arg_psString){
	bool	bRc;
	if (appType >= MQFB_APPL_FIRST && appType <= MQFB_APPL_LAST){
		ostringstream	ost;
		ost << "Appl value: " << appType;
		*arg_psString = ost.str();
		bRc =  false;
	}else{
		bRc = numberToString((char*)&CT_MQFB,sizeof(CT_MQFB),appType,arg_psString);
	}
	return bRc;
}
long X_Mq::convertToNumberMQFB(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQFB,sizeof(CT_MQFB),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQFB(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQFB,sizeof(CT_MQFB),pReason,sString);
}
//****************************************************************************************************
// MQMF conversions
//****************************************************************************************************
bool X_Mq::convertToStringMQMF(long arg_lNumber,string *arg_psString){
	bool	bRc = false;
	string s;

	if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_NONE,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;	
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_SEGMENTATION_ALLOWED,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_LAST_SEGMENT,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}else{
		if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_SEGMENT,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}
	}
	
	if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_LAST_MSG_IN_GROUP,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}else{
		if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_MSG_IN_GROUP,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}
	}
	
	if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_REJECT_UNSUP_MASK,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_ACCEPT_UNSUP_IF_XMIT_MASK,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQMF,sizeof(CT_MQMF),arg_lNumber,MQMF_ACCEPT_UNSUP_MASK,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}

	if ( (*arg_psString)[arg_psString->size()-1] == XMQ_BITWISE_DELIMITER){
		arg_psString->resize(arg_psString->size()-1);
	}

	return bRc;
}

string X_Mq::convertToStringMQMF(long arg_lNumber){

	string s;
	convertToStringMQMF(arg_lNumber,&s);

	return s;

}
long X_Mq::convertToNumberMQMF(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQMF,sizeof(CT_MQMF),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQMF(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQMF,sizeof(CT_MQMF),pReason,sString);
}


//****************************************************************************************************
// MQENC conversions
//****************************************************************************************************
bool X_Mq::convertToStringMQENC(long arg_lNumber,string *arg_psString){
	bool	bRc = false;
	string s;

	if (arg_lNumber == MQENC_NATIVE){
		*arg_psString = "MQENC_NATIVE: ";
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_INTEGER_NORMAL,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_INTEGER_REVERSED,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_INTEGER_MASK,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_DECIMAL_NORMAL,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_DECIMAL_REVERSED,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}
	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_DECIMAL_MASK,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}


	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_FLOAT_S390,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}else{
		if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_FLOAT_IEEE_NORMAL,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}
		if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_FLOAT_IEEE_REVERSED,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}
	}
	
	if (convertBitwiseNumberToString((char*)&CT_MQENC,sizeof(CT_MQENC),arg_lNumber,MQENC_FLOAT_MASK,&s)){
		*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
		bRc = true;
	}

	if ( (*arg_psString)[arg_psString->size()-1] == XMQ_BITWISE_DELIMITER){
		arg_psString->resize(arg_psString->size()-1);
	}


	return bRc;
}

string X_Mq::convertToStringMQENC(long arg_lNumber){

	string s;
	convertToStringMQENC(arg_lNumber,&s);

	return s;

}
long X_Mq::convertToNumberMQENC(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQENC,sizeof(CT_MQENC),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQENC(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQENC,sizeof(CT_MQENC),pReason,sString);
}

//****************************************************************************************************
// MQCHS conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQCHS(long	chanStatus){
	string s;
	numberToString((char*)&CT_MQCHS,sizeof(CT_MQCHS),chanStatus,&s);
	return s;
}

bool	X_Mq::convertToStringMQCHS(long	chanStatus,string* arg_psString){
	return numberToString((char*)&CT_MQCHS,sizeof(CT_MQCHS),chanStatus,arg_psString);
}
long X_Mq::convertToNumberMQCHS(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQCHS,sizeof(CT_MQCHS),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQCHS(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQCHS,sizeof(CT_MQCHS),pReason,sString);
}

//****************************************************************************************************
// MQCACH conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQCACH(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQCACH,sizeof(CT_MQCACH),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQCACH(long	chanStatus,string* arg_psString){
	return numberToString((char*)&CT_MQCACH,sizeof(CT_MQCACH),chanStatus,arg_psString);
}
long X_Mq::convertToNumberMQCACH(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQCACH,sizeof(CT_MQCACH),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQCACH(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQCACH,sizeof(CT_MQCACH),pReason,sString);
}

//****************************************************************************************************
// MQCA conversions
//****************************************************************************************************
bool	X_Mq::convertToStringMQCA(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQCA,sizeof(CT_MQCA),arg_lNumber,arg_psString);
}

string	X_Mq::convertToStringMQCA(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQCA,sizeof(CT_MQCA),arg_lNumber,&s);
	return s;
}
long X_Mq::convertToNumberMQCA(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQCA,sizeof(CT_MQCA),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQCA(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQCA,sizeof(CT_MQCA),pReason,sString);
}

//****************************************************************************************************
// MQIA conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQIA(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQIA,sizeof(CT_MQIA),arg_lNumber,&s);
	return s;
}

bool	X_Mq::convertToStringMQIA(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQIA,sizeof(CT_MQIA),arg_lNumber,arg_psString);
}

long X_Mq::convertToNumberMQIA(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQIA,sizeof(CT_MQIA),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQIA(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQIA,sizeof(CT_MQIA),pReason,sString);
}

//****************************************************************************************************
// MQCACF conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQCACF(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQCACF,sizeof(CT_MQCACF),arg_lNumber,&s);
	return s;
}

bool	X_Mq::convertToStringMQCACF(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQCACF,sizeof(CT_MQCACF),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQCACF(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQCACF,sizeof(CT_MQCACF),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQCACF(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQCACF,sizeof(CT_MQCACF),pReason,sString);
}

//****************************************************************************************************
// MQIACH conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQIACH(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQIACH,sizeof(CT_MQIACH),arg_lNumber,&s);
	return s;
}

bool	X_Mq::convertToStringMQIACH(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQIACH,sizeof(CT_MQIACH),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQIACH(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQIACH,sizeof(CT_MQIACH),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQIACH(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQIACH,sizeof(CT_MQIACH),pReason,sString);
}

//****************************************************************************************************
// MQIACF conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQIACF(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQIACF,sizeof(CT_MQIACF),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQIACF(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQIACF,sizeof(CT_MQIACF),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQIACF(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQIACF,sizeof(CT_MQIACF),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQIACF(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQIACF,sizeof(CT_MQIACF),pReason,sString);
}

//****************************************************************************************************
// MQRO conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQRO(long	arg_lNumber){
	string s;
	convertToStringMQRO(arg_lNumber,&s);
	return s;
}


bool	X_Mq::convertToStringMQRO(long	arg_lNumber,string* arg_psString){
	string s;
	bool	bRc = false;

	if (arg_lNumber == MQRO_NONE){
		*arg_psString = "MQRO_NEW_MSGID";
		*arg_psString += XMQ_BITWISE_DELIMITER ;
		*arg_psString+= "MQRO_COPY_MSG_ID_TO_CORREL_ID";
		*arg_psString += XMQ_BITWISE_DELIMITER;
		*arg_psString += "MQRO_DEAD_LETTER_Q";
		bRc = true;
	}else{
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_EXCEPTION,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_EXCEPTION_WITH_DATA,&s)){     
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_EXCEPTION_WITH_FULL_DATA,&s)){     
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		} 
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_EXPIRATION,&s)){        
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}            
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_EXPIRATION_WITH_DATA,&s)){      
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}    
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_EXPIRATION_WITH_FULL_DATA,&s)){     
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_COA,&s)){       
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}                    
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_COA_WITH_DATA,&s)){          
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}        
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_COA_WITH_FULL_DATA,&s)){       
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}     
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_COD,&s)){      
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}                     
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_COD_WITH_DATA,&s)){            
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}     
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_COD_WITH_FULL_DATA,&s)){         
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}   
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_PAN,&s)){        
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}                   
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_NAN,&s)){       
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}      
	
		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_PASS_MSG_ID,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}else{
			*arg_psString += "MQRO_NEW_MSGID";*arg_psString += XMQ_BITWISE_DELIMITER;
		}

		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_PASS_CORREL_ID,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}else{
			*arg_psString += "MQRO_COPY_MSG_ID_TO_CORREL_ID";*arg_psString += XMQ_BITWISE_DELIMITER;
		}

		if (convertBitwiseNumberToString((char*)&CT_MQRO,sizeof(CT_MQRO),arg_lNumber,MQRO_DISCARD_MSG,&s)){
			*arg_psString+=s;*arg_psString +=  XMQ_BITWISE_DELIMITER;
			bRc = true;
		}else{
			*arg_psString += "MQRO_DEAD_LETTER_Q";*arg_psString += XMQ_BITWISE_DELIMITER;
		}

	}

	if ( (*arg_psString)[arg_psString->size()-1] == XMQ_BITWISE_DELIMITER){
		arg_psString->resize(arg_psString->size()-1);
	}

	return bRc;
}
long X_Mq::convertToNumberMQRO(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQRO,sizeof(CT_MQRO),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQRO(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQRO,sizeof(CT_MQRO),pReason,sString);
}

//****************************************************************************************************
// MQPMO conversions
//****************************************************************************************************
long X_Mq::convertToNumberMQPMO(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQPMO,sizeof(CT_MQPMO),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQPMO(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQPMO,sizeof(CT_MQPMO),pReason,sString);
}

//****************************************************************************************************
// MQGMO conversions
//****************************************************************************************************
long X_Mq::convertToNumberMQGMO(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQGMO,sizeof(CT_MQGMO),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQGMO(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQGMO,sizeof(CT_MQGMO),pReason,sString);
}


//****************************************************************************************************
// MQRQ conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQRQ(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQRQ,sizeof(CT_MQRQ),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQRQ(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQRQ,sizeof(CT_MQRQ),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQRQ(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQRQ,sizeof(CT_MQRQ),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQRQ(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQRQ,sizeof(CT_MQRQ),pReason,sString);
}

//****************************************************************************************************
// MQCFT conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQCFT(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQCFT,sizeof(CT_MQCFT),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQCFT(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQCFT,sizeof(CT_MQCFT),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQCFT(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQCFT,sizeof(CT_MQCFT),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQCFT(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQCFT,sizeof(CT_MQCFT),pReason,sString);
}

//****************************************************************************************************
// MQUS conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQUS(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQUS,sizeof(CT_MQUS),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQUS(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQUS,sizeof(CT_MQUS),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQUS(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQUS,sizeof(CT_MQUS),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQUS(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQUS,sizeof(CT_MQUS),pReason,sString);
}

//****************************************************************************************************
// MQTT conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQTT(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQTT,sizeof(CT_MQTT),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQTT(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQTT,sizeof(CT_MQTT),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQTT(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQTT,sizeof(CT_MQTT),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQTT(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQTT,sizeof(CT_MQTT),pReason,sString);
}

//****************************************************************************************************
// MQPL conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQPL(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQPL,sizeof(CT_MQPL),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQPL(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQPL,sizeof(CT_MQPL),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQPL(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQPL,sizeof(CT_MQPL),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQPL(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQPL,sizeof(CT_MQPL),pReason,sString);
}

//****************************************************************************************************
// MQXPT conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQXPT(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQXPT,sizeof(CT_MQXPT),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQXPT(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQXPT,sizeof(CT_MQXPT),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQXPT(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQXPT,sizeof(CT_MQXPT),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQXPT(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQXPT,sizeof(CT_MQXPT),pReason,sString);
}

//****************************************************************************************************
// MQQDT conversions
//****************************************************************************************************
string	X_Mq::convertToStringMQQDT(long	arg_lNumber){
	string s;
	numberToString((char*)&CT_MQQDT,sizeof(CT_MQQDT),arg_lNumber,&s);
	return s;
}
bool	X_Mq::convertToStringMQQDT(long	arg_lNumber,string* arg_psString){
	return numberToString((char*)&CT_MQQDT,sizeof(CT_MQQDT),arg_lNumber,arg_psString);
}
long X_Mq::convertToNumberMQQDT(string s){
	long r = 0;
	stringToNumber((char*)&CT_MQQDT,sizeof(CT_MQQDT),&r,s);
	return r;
}
bool X_Mq::convertToNumberMQQDT(long* pReason,string sString){
	return stringToNumber((char*)&CT_MQQDT,sizeof(CT_MQQDT),pReason,sString);
}

//****************************************************************************************************
// sleep function (no ANSI C/C++ sleep function)
//****************************************************************************************************
bool	X_Mq::sleep(long arg_lSleepTime,ImqQueueManager* arg_pQM){

	if (!arg_pQM || !arg_pQM->connectionStatus() || arg_lSleepTime == 0){
		return false;
	}

	ImqQueue	q;

	q.setName("SYSTEM.DEFAULT.MODEL.QUEUE");
	q.setConnectionReference(arg_pQM);
	q.setOpenOptions(MQOO_INQUIRE | MQOO_INPUT_SHARED);

	if (!q.open()){
		//do nothing
	}else{
		ImqGetMessageOptions getMsgOptions;
		ImqMessage				md;

		getMsgOptions.setMatchOptions(MQMO_MATCH_MSG_ID);
		getMsgOptions.setOptions(MQGMO_CONVERT||MQGMO_WAIT);
		getMsgOptions.setWaitInterval(arg_lSleepTime);

		if (!q.get(md,getMsgOptions)){
			//do nothing...
		}
		q.close();
	}


	return true;
	
} 

string	X_Mq::getVersion(){
	return X_MQ_VERSION;
}
